@@ -16,13 +16,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: bigint.h,v 1.1 2001/11/01 18:27:27 joey Exp $
+    $Id: bigint.h,v 1.3 2005-08-26 05:12:28 joey Exp $
  */
 
 /*
  * 4294967295 is max (unsigned long int)
  */
 
+#ifndef _BIGINT_H_
+#define _BIGINT_H_
+
 #define BIGINT_LEN	2
 #define BIGINT_MAX	1000000000
 #define BIGINT_PREC	9
@@ -41,3 +44,12 @@ bigint bigint_int(int num);
 bigint bigint_string(char *s);
 
 void bigint_sprintf (char *res, char *format, bigint a);
+
+/* compares two bigints
+   a < b  => -1
+   a > b  =>  1
+   a == b =>  0
+*/
+int bigint_cmp(bigint a, bigint b);
+
+#endif /* _BIGINT_H_ */
